@@ -92,7 +92,7 @@ module Control.Concurrent.Async (
     -- ** Spawning with automatic 'cancel'ation
     withAsync, withAsyncBound, withAsyncOn, withAsyncWithUnmask, withAsyncOnWithUnmask,
 
-    -- ** Quering 'Async's
+    -- ** Querying 'Async's
     wait, poll, waitCatch, cancel, cancelWith,
     asyncThreadId,
 
@@ -246,7 +246,10 @@ wait = atomically . waitSTM
 --
 {-# INLINE waitCatch #-}
 waitCatch :: Async a -> IO (Either SomeException a)
-waitCatch = atomically . waitCatchSTM
+waitCatch = tryAgain . atomically . waitCatchSTM
+  where
+    -- See: https://github.com/simonmar/async/issues/14
+    tryAgain f = f `catch` \BlockedIndefinitelyOnSTM -> f
 
 -- | Check whether an 'Async' has completed yet.  If it has not
 -- completed yet, then the result is @Nothing@, otherwise the result
@@ -568,6 +571,11 @@ instance Alternative Concurrently where
   Concurrently as <|> Concurrently bs =
     Concurrently $ either id id <$> race as bs
 
+instance Monad Concurrently where
+  return = pure
+  Concurrently a >>= f =
+    Concurrently $ a >>= runConcurrently . f
+
 -- ----------------------------------------------------------------------------
 
 -- | Fork a thread that runs the supplied action, and if it raises an
